#include <stdio.h>
#include <math.h>

#include <maxent.h>

int main()
{

  /* First declare a set of 2-dimensional moment constraints of up to
   * 4th order. Altogether we will have 15 constraints. */

  moments m = moment_set_dim_pow(2,4);

  /* Second, declare a set of multipliers. */

  multipliers p = multiplier_init();

  /* Third, instantiate the maximum entropy routine object. In this
   * example it will have 50-point Gauss-Hermite quadrature. */

  maxent mxnt = maxent_set_quad_size(50);

  /* Now define 16 moment constraints for the problem. First, declare
   * an integer vector of length 2: */

  int i[2];

  /* In order to set the value for each moment in the moments class,
   * powers of all variables corresponding to the current moment have
   * to be provided, plus the value of the moment itself. In order to
   * provide powers in our 2D setting, we have just reserved i. i[0]
   * will provide the power of the first variable, and i[1] will
   * provide the power of the second variable. */

  i[0]=0; i[1]=0; /* Zero power constraint, supposed to be 1 for a PDF. */
  moment_set_value(m,i,1.0);

  /* Set up first powers (which are the mean states) */
  
  i[0]=1; i[1]=0; /* Mean of the first variable */
  moment_set_value(m,i,0.1);

  i[0]=0; i[1]=1; /* Mean of the second variable */
  moment_set_value(m,i,0.2);

  /* Set up second power moments (covariances), there are three of them */

  i[0]=2; i[1]=0; /* Variance of the first variable (x^2) */
  moment_set_value(m,i,0.9);

  i[0]=1; i[1]=1; /* Covariance between the two variables (xy) */
  moment_set_value(m,i,0.2);

  i[0]=0; i[1]=2; /* Variance of the first variable (y^2) */
  moment_set_value(m,i,1.1);


  /* Set up third power moments (skewness), there are four of them */

  i[0]=3; i[1]=0; /* Skewness of the first variable (x^3) */
  moment_set_value(m,i,0.2);

  i[0]=2; i[1]=1; /* Cross-skewness between the two variables (x^2y) */
  moment_set_value(m,i,-0.05);

  i[0]=1; i[1]=2; /* Another cross-skewness between the two variables
		   * (xy^2) */
  moment_set_value(m,i,0.1);

  i[0]=0; i[1]=3; /* Skewness of the second variable (y^3) */
  moment_set_value(m,i,-0.3);


  /* Set up fourth power moments (flatness), there are five of them */

  i[0]=4; i[1]=0; /* Flatness of the first variable (x^4) */
  moment_set_value(m,i,1.8);

  i[0]=3; i[1]=1; /* Cross-flatness between the two variables (x^3y) */
  moment_set_value(m,i,0.03);

  i[0]=2; i[1]=2; /* Another cross-flatness between the two variables
		   * (x^2y^2) */
  moment_set_value(m,i,0.8);

  i[0]=1; i[1]=3; /* Another cross-flatness between the two variables
		   * (xy^3) */
  moment_set_value(m,i,0.04);

  i[0]=0; i[1]=4; /* Flatness of the second variable (y^4) */
  moment_set_value(m,i,2.0);
  
  /* Checking contents of m */

  puts("------------- Contents of m -------------");
  moment_print(m,stdout);

  int status = maxent_compute(mxnt,m,p);

  /* Status is nonzero if errors have been encountered (i.e. the
   * algorithm did not converge) */

  if(status) puts("Error in maxent");
  else puts("No error in maxent");

  /* p now contains the values of multipliers, if no error has
   * happened. */

  /* Checking contents of p */

  puts("------------- Contents of p -------------");
  multiplier_print(p,stdout);

  /* The Lagrange multipliers of p are obtained in a similar manner as
   * the constraints are set for m. Namely: */

  i[0]=2; i[1]=1;
  double mult21 = multiplier_get_value(p,i);

  /* mult21 now has the value of the Lagrange multiplier which
   * corresponds to the x^2y power. One can retrieve all the values of
   * the Lagrange multipliers in the same manner. */

  printf("The Lagrange multiplier for x^2y is %g\n", mult21);

  /* Here is another way to get the shape of the PDF. If one provides
   * a floating-point vector x to p, it will return the value of the
   * polynomial consisting of Lagrange multipliers and powers at
   * x. One can do it repeatedly for multiple coordinates. Below is an
   * example of how to retrieve values of the computed PDF on a 61x61
   * points 2D-domain for -3<x<3 and -3<y<3. */

  int ix, iy, Nx=61, Ny=61;
  double limx = 3.0, limy = 3.0;
  double x[2];
  double scale[Nx];
  double PDF_values[Nx*Ny];

  double deltax = 2.0*limx/(Nx-1);
  double deltay = 2.0*limy/(Ny-1);

  for(iy=0; iy<Ny; ++iy){
    x[1] = -limy + deltay*iy;
    scale[iy] = x[1]; /* Since the scales in x and y are identical,
                       * using the same for both */
    for(ix=0; ix<Nx; ++ix){
      x[0] = -limx + deltax*ix;
      PDF_values[iy*Nx+ix] = exp(multiplier_get_poly_value(p,x));
      /* PDF is an exponent of the polynomial */

    }
  }

  /* Now the values of the maxent PDF are written into the PDF_values
   * array. Here is an example of how to output then in the gnuplot
   * format */

  FILE * output = fopen("maxent_example_2D_c.dat","wb");

  for (ix=0; ix<Nx; ++ix){
    for(iy=0; iy<Ny; ++iy)
      fprintf(output,"%g\t%g\t%g\n",scale[ix],scale[iy],PDF_values[iy*Nx+ix]);
    fputs("\n",output);
  }

  fclose(output);

  maxent_free(mxnt);
  multiplier_free(p);
  moment_free(m);

  /* Typing 'splot "maxent_example_2D_c.dat" with lines' at the
   * gnuplot prompt will produce the 3D mesh plot with this example
   * PDF. */
  
  return 0;
  
}
