#include <iostream>
#include <fstream>
#include <cmath>

#include <maxent.hh>

using namespace std;

int main()
{

  // First declare a set of 2-dimensional moment constraints of up to
  // 4th order. Altogether we will have 15 constraints.

  moments m(2,4);

  // Second, declare a set of multipliers.

  multipliers p;

  // Third, instantiate the maximum entropy routine object. In this
  // example it will have 50-point Gauss-Hermite quadrature.

  maxent compute_maxent(50);

  // Now define 16 moment constraints for the problem. First, declare
  // an integer vector of length 2:

  int i[2];

  // In order to set the value for each moment in the moments class,
  // powers of all variables corresponding to the current moment have
  // to be provided, plus the value of the moment itself. In order to
  // provide powers in our 2D setting, we have just reserved x. x[0]
  // will provide the power of the first variable, and x[1] will
  // provide the power of the second variable. Here is the example:

  i[0]=0; i[1]=0; // Zero power constraint, supposed to be 1 for a PDF.
  m(i)=1;

  // Set up first powers (which are the mean states)
  
  i[0]=1; i[1]=0; // Mean of the first variable
  m(i)=0.1;

  i[0]=0; i[1]=1; // Mean of the second variable
  m(i)=0.2;

  // Set up second power moments (covariances), there are three of them

  i[0]=2; i[1]=0; // Variance of the first variable (x^2)
  m(i)=0.9;

  i[0]=1; i[1]=1; // Covariance between the two variables (xy)
  m(i)=0.2;

  i[0]=0; i[1]=2; // Variance of the first variable (y^2)
  m(i)=1.1;


  // Set up third power moments (skewness), there are four of them

  i[0]=3; i[1]=0; // Skewness of the first variable (x^3)
  m(i)=0.2;

  i[0]=2; i[1]=1; // Cross-skewness between the two variables (x^2y)
  m(i)=-0.05;

  i[0]=1; i[1]=2; // Another cross-skewness between the two variables
		  // (xy^2)
  m(i)=0.1;

  i[0]=0; i[1]=3; // Skewness of the second variable (y^3)
  m(i)=-0.3;


  // Set up fourth power moments (flatness), there are five of them

  i[0]=4; i[1]=0; // Flatness of the first variable (x^4)
  m(i)=1.8;

  i[0]=3; i[1]=1; // Cross-flatness between the two variables (x^3y)
  m(i)=0.03;

  i[0]=2; i[1]=2; // Another cross-flatness between the two variables
		  // (x^2y^2)
  m(i)=0.8;

  i[0]=1; i[1]=3; // Another cross-flatness between the two variables
		  // (xy^3)
  m(i)=0.04;

  i[0]=0; i[1]=4; // Flatness of the second variable (y^4)
  m(i)=2.0;
  
  // Checking contents of m

  cout << "------------- Contents of m -------------" << endl;
  m.print();

  int status = compute_maxent(m,p);

  // Status is nonzero if errors have been encountered (i.e. the
  // algorithm did not converge)

  if(status) cout << "Error in maxent" << endl;
  else cout << "No error in maxent" << endl;

  // p now contains the values of multipliers, if no error has
  // happened.

  // Checking contents of p

  cout << "------------- Contents of p -------------" << endl;
  p.print();

  // The Lagrange multipliers of p are obtained in a similar manner as
  // the constraints are set for m. Namely:

  i[0]=2; i[1]=1;
  double mult21 = p(i);

  // mult21 now has the value of the Lagrange multiplier which
  // corresponds to the x^2y power. One can retrieve all the values of
  // the Lagrange multipliers in the same manner.

  cout << "The Lagrange multiplier for x^2y is " << mult21 << endl;

  // Here is another way to get the shape of the PDF. If one provides
  // a floating-point vector x to p, it will return the value of the
  // polynomial consisting of Lagrange multipliers and powers at
  // x. One can do it repeatedly for multiple coordinates. Below is an
  // example of how to retrieve values of the computed PDF on a 61x61
  // points 2D-domain for -3<x<3 and -3<y<3.

  int ix, iy, Nx=61, Ny=61;
  double limx = 3.0, limy = 3.0;
  double x[2];
  double scale[Nx];
  double PDF_values[Nx*Ny];

  double deltax = 2.0*limx/(Nx-1);
  double deltay = 2.0*limx/(Ny-1);

  for(iy=0; iy<Ny; ++iy){
    x[1] = -limy + deltay*iy;
    scale[iy] = x[1]; // Since the scales in x and y are identical,
                      // using the same for both
    for(ix=0; ix<Nx; ++ix){
      x[0] = -limx + deltax*ix;
      PDF_values[iy*Nx+ix] = exp(p(x)); // PDF is an exponent of the
                                        // polynomial

    }
  }

  // Now the values of the maxent PDF are written into the PDF_values
  // array. Here is an example of how to output then in the gnuplot
  // format

  ofstream output("maxent_example_2D.dat", ios::out);

  for (ix=0; ix<Nx; ++ix){
    for(iy=0; iy<Ny; ++iy)
      output << scale[ix] << "\t" << scale[iy] << "\t"
	     << PDF_values[iy*Nx+ix] << endl;
    output << endl;
  }

  // Typing 'splot "maxent_example_2D.dat" with lines' at the gnuplot
  // prompt will produce the 3D mesh plot with this example PDF.
  
}
